@@ -13,8 +13,7 @@ from typing import (
 
 from transformers.tokenization_utils_base import PreTrainedTokenizerBase
 
-from ..base.mappers import BatchedBaseMapper, SingleBaseMapper
-from ..base.types import TransformElementType
+from ..base import BatchedBaseMapper, SingleBaseMapper, TransformElementType
 
 
 class TokensSequencesPaddingMapper(SingleBaseMapper):
